@@ -605,18 +605,14 @@ static CACHE_TABLE *getEmptyCache(int din,size_t begin)
  */
 static void start_delay_write_task()
 {
-	int proc;
-
-
-	switch(proc=sys_fork())
+	switch(sys_fork())
     {
 		case -1:
 			printk("Faild start_delay_write_task\n");
 			break;
 		case 0:
+			write_task=get_current_task();
 			delay_write();
-		default:
-			write_task=(PROC*)proc;
 	}
 }
 
@@ -1129,8 +1125,20 @@ static int readdir(int din,uint inode,DIR *dir,char *name)
 }
 
 
-static int stat(int din,uint dir_inode,FSTAT *fstat)
+static int stat(int din,uint inode,FSTAT *fstat)
 {
+	fstat->st_dev=inode;
+	fstat->st_ino=0;
+	fstat->st_mode=0;
+	fstat->st_nlink=0;
+	fstat->st_uid=0;
+	fstat->st_gid=0;
+	fstat->st_size=dinode[inode].blocks*dinode[inode].dev->sector_size;
+	fstat->st_atime=0;
+	fstat->st_mtime=0;
+	fstat->st_ctime=0;
+	fstat->st_blksize=dinode[inode].dev->sector_size;
+
 	return 0;
 }
 
@@ -1138,7 +1146,7 @@ static int stat(int din,uint dir_inode,FSTAT *fstat)
 /*
  * parameters : device inode,inode,Command,Parameter struct pointer
  */
-static int ioctl(int din,int prm_num,int cmd,...)
+static int ioctl(int din,uint inode,int prm_num,int cmd,...)
 {
 	uint *arg=(uint*)&cmd+1;
 
@@ -1146,11 +1154,11 @@ static int ioctl(int din,int prm_num,int cmd,...)
 	switch(prm_num)
 	{
 		case 0:
-			return dinode[din].dev->ioctl(0,cmd);
+			return dinode[inode].dev->ioctl(0,cmd);
 		case 1:
-			return dinode[din].dev->ioctl(1,cmd,*arg);
+			return dinode[inode].dev->ioctl(1,cmd,*arg);
 		case 2:
-			return dinode[din].dev->ioctl(2,cmd,*arg,*(arg+1));
+			return dinode[inode].dev->ioctl(2,cmd,*arg,*(arg+1));
 		default:
 			return 0;
 	}
