@@ -2575,6 +2575,7 @@ static int stat(int din,uint inode,FSTAT *fstat)
 	fstat->st_atime=((INODE*)inode)->i_atime;
 	fstat->st_mtime=((INODE*)inode)->i_mtime;
 	fstat->st_ctime=((INODE*)inode)->i_ctime;
+	fstat->st_blksize=1024<<superBlock[din]->s_log_block_size;
 
 	return 0;
 }
@@ -2693,7 +2694,7 @@ ERR:
 }
 
 
-static int ioctl(int din,int prm_num,int cmd,...)
+static int ioctl(int din,uint inode,int prm_num,int cmd,...)
 {
 
 	return 0;
