@@ -53,7 +53,7 @@ static struct ROOTENT{
 	VENTRY vent;
 	char name[1];
 }_rootEnt={
-	{(VENTRY*)&_rootEnt,(VENTRY*)&_rootEnt,0,(VENTRY*)&_rootEnt,NULL,NULL,NULL,0,NULL,1,0,DIRECTORY,1},
+	{NULL,0,(VENTRY*)&_rootEnt,(VENTRY*)&_rootEnt,(VENTRY*)&_rootEnt,NULL,NULL,0,NULL,1,0,DIRECTORY,1},
 	{'/'},
 };
 static VENTRY *rootEnt=(VENTRY*)&_rootEnt;
@@ -63,7 +63,7 @@ static struct DEVENT{
 	VENTRY vent;
 	char name[3];
 }_devEnt={
-	{(VENTRY*)&_devEnt,(VENTRY*)&_devEnt,0,(VENTRY*)&_rootEnt,NULL,NULL,NULL,0,NULL,1,0,DIRECTORY,3},
+	{NULL,0,(VENTRY*)&_devEnt,(VENTRY*)&_devEnt,(VENTRY*)&_rootEnt,NULL,NULL,0,NULL,1,0,DIRECTORY,3},
 	{'d','e','v'},
 };
 static VENTRY *devEnt=(VENTRY*)&_devEnt;
@@ -455,7 +455,7 @@ static FS fsInfoTop;
  * parameters : file system name
  * return : FS address or NULL
  */
-static inline FS *search_fs(const char *name)
+static FS *search_fs(const char *name)
 {
 	FS *p;
 
@@ -573,22 +573,6 @@ static inline void setCurrentDir(VENTRY *vent)
 }
 
 
-/*
- * PUBLIC
- * ץեγǧ
- * return : 0 or error number
- */
-static inline int checkOpenNum()
-{
-	FILE_STRUCT *fstruct;
-
-
-	fstruct=get_current_task()->file_struct;
-	if(fstruct->fd_count>=MAX_FILE_OPEN)return -EMFILE;
-	else return 0;
-}
-
-
 /*
  * PUBLIC
  * Get file descriptor flag.
@@ -669,37 +653,6 @@ static inline int setOpenFlag(int num,int flag)
 }
 
 
-/*
- * PUBLIC
- * եǥץϿ롣
- * parameters : ventry,access mode,fd flag
- * return : file descriptor number or error number
- */
-static int setFd(VENTRY *vent,int amode,int flag)
-{
-	int num;
-	FILE_STRUCT *fstruct;
-	F_DSC *fd;
-
-
-	fstruct=get_current_task()->file_struct;
-	if((fd=kmalloc(sizeof(F_DSC)))==NULL)return -ENOMEM;
-
-	/* եǥץν */
-	fd->form=FD_FORM_LOCALFILE;
-	fd->accessMode=amode;
-	fd->offset=0;
-	fd->refCount=1;
-	fd->vent=vent;
-	num=getEmtpyFdNum(fstruct,fstruct->next_fd);
-	++fstruct->fd_count;
-	fstruct->fd[num]=fd;
-	fstruct->flag[num]=flag;
-
-	return num;
-}
-
-
 /*
  * PUBLIC
  * Copy file descriptor.
@@ -904,21 +857,26 @@ int init_file_struct(PROC *proc)
 
 /*
  * GLOBAL
- * Set file descriptar,for network socket.
- * parameters : file descriptor
- * return : file descriptar number or error number
+ * եǥץϿ롣
+ * parameters : ventry,file format,access mode,fd flag,F_DSC buffer
+ * return : file descriptor number or error number
  */
-int setToFd(void *fd)
+int setFd(VENTRY *vent,int form,int amode,int flag,F_DSC *fd)
 {
 	int num;
-	FILE_STRUCT *fstruct;
+	FILE_STRUCT *fstruct=get_current_task()->file_struct;
 
 
-	fstruct=get_current_task()->file_struct;
-	if(fstruct->fd_count>=MAX_FILE_OPEN)return -EMFILE;
+	/* եǥץν */
+	fd->form=form;
+	fd->accessMode=amode;
+	fd->offset=0;
+	fd->refCount=1;
+	fd->vent=vent;
 	num=getEmtpyFdNum(fstruct,fstruct->next_fd);
-	fstruct->fd[num]=fd;
 	++fstruct->fd_count;
+	fstruct->fd[num]=fd;
+	fstruct->flag[num]=flag;
 
 	return num;
 }
@@ -937,6 +895,22 @@ F_DSC *getFd(int num)
 }
 
 
+/*
+ * GLOBAL
+ * ץեγǧ
+ * return : 0 or error number
+ */
+int checkFileOpen()
+{
+	FILE_STRUCT *fstruct;
+
+
+	fstruct=get_current_task()->file_struct;
+	if(fstruct->fd_count>=MAX_FILE_OPEN)return -EMFILE;
+	else return 0;
+}
+
+
 /************************************************************************************
  *
  * System call interface
@@ -984,7 +958,7 @@ int exec_open(const char *path,OPEN_F *open_f)
 	uint inode;
 	int rest;
 	SEARCH_PARAM start,end;
-	VENTRY *vent;
+/*	VENTRY *vent;*/
 
 
 	if(*path=='\0')return -ENOENT;
@@ -1019,32 +993,34 @@ int exec_open(const char *path,OPEN_F *open_f)
 
 	if(*end.path=='\0')
 	{
-		linkVentry(end.vent);
-		vent=end.vent;
+/*		linkVentry(end.vent);*/		/* ۥեؤΥ󥯤α */
+		open_f->din=end.vent->din;
+		open_f->inode=end.vent->block;
+		open_f->fs=end.vent->fs;
 	}
 	else
 	{
 		if((rest=start.vent->fs->open(start.path,start.vent->din,start.vent->block,0,&inode))<0)
 			goto END;
 
-		enter_spinlock(&spinLock);
+		/* ۥեκα */
+/*		enter_spinlock(&spinLock);
 		{
 			rest=mkEntry(end.path,end.vent);
 		}
 		exit_spinlock(&spinLock);
-
+*/
 		if(rest<0)
 		{
 			start.vent->fs->close(start.vent->din,start.vent->block);
 			goto END;
 		}
 
-		vent=(VENTRY*)rest;
-		vent->block=inode;
-		vent->type=NORMAL_FILE;
+		open_f->din=start.vent->din;
+		open_f->inode=inode;
+		open_f->fs=start.vent->fs;
 	}
 
-	open_f->vent=vent;
 	rest=0;
 END:
 	resetBusy(end.vent);
@@ -1061,11 +1037,7 @@ END:
  */
 int exec_read(OPEN_F *open_f,void *buf,size_t size)
 {
-	VENTRY *vent;
-
-
-	vent=open_f->vent;
-	return vent->fs->read(vent->din,vent->block,buf,size,open_f->offset);
+	return open_f->fs->read(open_f->din,open_f->inode,buf,size,open_f->offset);
 }
 
 
@@ -1211,7 +1183,7 @@ int sys_lseek(int fd_num,int offset,int flag)
 
 	if((fd=getFd(fd_num))==NULL)return -EBADF;
 	vent=fd->vent;
-	vent->fs->stat(0,vent->block,&fstat);
+	vent->fs->stat(vent->din,vent->block,&fstat);
 	switch(flag)
 	{
 		case SEEK_CUR:
@@ -1237,12 +1209,13 @@ int sys_creat(const char *path,int flag,int mode)
 	int block;
 	SEARCH_PARAM start,end;
 	VENTRY *vent;
+	F_DSC *fd;
 
 
 	if(*path=='\0')return -ENOENT;
 
 	/* ץΥå */
-	if((rest=checkOpenNum())<0)return rest;
+	if((rest=checkFileOpen())<0)return rest;
 
 	/* եõơۥȥ꡼롣 */
 	start.vent=getTopDir(&path);
@@ -1299,7 +1272,12 @@ int sys_creat(const char *path,int flag,int mode)
 	else vent->type=NORMAL_FILE;
 
 	/* եǥץν */
-	rest=setFd(vent,flag,FD_CLOEXEC);
+	if((fd=kmalloc(sizeof(F_DSC)))==NULL)
+	{
+		rest=-ENOMEM;
+		goto END;
+	}
+	rest=setFd(vent,FD_FORM_LOCALFILE,flag,FD_CLOEXEC,fd);
 END:
 	resetBusy(end.vent);
 
@@ -1317,12 +1295,13 @@ int sys_open(const char *path,int flag,int mode)
 	int rest;
 	SEARCH_PARAM start,end;
 	VENTRY *vent;
+	F_DSC *fd;
 
 
 	if((flag&(O_CREAT|O_EXCL))==(O_CREAT|O_EXCL))return sys_creat(path,flag,mode);
 
 	/* ץΥå */
-	if((rest=checkOpenNum())<0)return rest;
+	if((rest=checkFileOpen())<0)return rest;
 
 	/* եõơۥȥ꡼롣 */
 	start.vent=getTopDir(&path);
@@ -1392,7 +1371,12 @@ int sys_open(const char *path,int flag,int mode)
 	}
 
 	/* եǥץϿ롣 */
-	if((rest=setFd(vent,mode,FD_CLOEXEC))<0)goto END;
+	if((fd=kmalloc(sizeof(F_DSC)))==NULL)
+	{
+		rest=-ENOMEM;
+		goto END;
+	}
+	rest=setFd(vent,FD_FORM_LOCALFILE,mode,FD_CLOEXEC,fd);
 	if(flag&O_APPEND)sys_lseek(rest,0,SEEK_END);
 END:
 	resetBusy(end.vent);
@@ -1430,6 +1414,7 @@ int sys_read(int fd_num,void *buf,size_t size)
 
 
 	if((fd=getFd(fd_num))==NULL)return -EBADF;
+	if(checkMem(buf)==-1)return -EFAULT;
 	if((fd->accessMode&(O_RDONLY|O_RDWR))==0)return -EBADF;		/* ⡼ɤΥå */
 
 	/* readؿƤӽФ */
@@ -1458,6 +1443,7 @@ int sys_write(int fd_num,void *buf,size_t size)
 
 
 	if((fd=getFd(fd_num))==NULL)return -EBADF;
+	if(checkMem(buf)==-1)return -EFAULT;
 	if((fd->accessMode&(O_WRONLY|O_RDWR))==0)return -EBADF;		/* ⡼ɤΥå */
 
 	/* writeؿƤӽФ */
@@ -1833,7 +1819,7 @@ int sys_stat(int fd_num,const char *path,struct stat *buf)
 	{
 		if((fd=getFd(fd_num))==NULL)return -EBADF;
 		buf->st_crtpos=fd->offset;
-		return fd->vent->fs->stat(0,fd->vent->block,buf);
+		return fd->vent->fs->stat(fd->vent->din,fd->vent->block,buf);
 	}
 	else
 	{
@@ -1843,7 +1829,7 @@ int sys_stat(int fd_num,const char *path,struct stat *buf)
 		start.path=path;
 		if((rest=searchPath(&start,&end))<0)return rest;
 		if((*end.path=='\0')&&(end.vent->block!=0))
-			return end.vent->fs->stat(0,end.vent->block,buf);
+			return end.vent->fs->stat(end.vent->din,end.vent->block,buf);
 		else
 		{
 			if((rest=start.vent->fs->open(start.path,start.vent->din,start.vent->block,0,&inode))==-EISDIR)
@@ -1852,7 +1838,7 @@ int sys_stat(int fd_num,const char *path,struct stat *buf)
 				inode=rest;
 			}
 			if(rest<0)return rest;
-			rest=start.vent->fs->stat(0,inode,buf);
+			rest=start.vent->fs->stat(start.vent->din,inode,buf);
 			start.vent->fs->close(start.vent->din,inode);
 
 			return rest;
@@ -1901,7 +1887,7 @@ int sys_fcntl(int fd_num,int cmd,uint arg)
 		{
 			int rest;
 
-			if((rest=checkOpenNum())<0)return rest;		/* ץΥå */
+			if((rest=checkFileOpen())<0)return rest;		/* ץΥå */
 			if(getFd(fd_num)==NULL)return -EBADF;		/* File descriptorΥå */
 			return copyFd(fd_num,arg);					/* Copy File descriptor. */
 		}
@@ -1929,6 +1915,7 @@ int sys_chattr(const char *path,int cmd,uint attr)
 	uint inode=0;
 	int rest;
 	int open=0;
+	int din;
 	SEARCH_PARAM start,end;
 	FS *fs;
 	PROC *proc;
@@ -1959,18 +1946,20 @@ int sys_chattr(const char *path,int cmd,uint attr)
 	if((*end.path=='\0')&&(end.vent->block!=0))
 	{
 		fs=end.vent->fs;
+		din=end.vent->din;
 		inode=end.vent->block;
 	}
 	else
 	{
 		fs=start.vent->fs;
+		din=start.vent->din;
 		if((rest=fs->open(start.path,start.vent->din,start.vent->block,0,&inode))<0)goto ERR;
 		open=1;
 	}
 
 	/* Change attribute. */
 	proc=get_current_task();
-	if((rest=fs->stat(0,inode,&stat))<0)goto ERR;
+	if((rest=fs->stat(din,inode,&stat))<0)goto ERR;
 	switch(cmd)
 	{
 		case CHATTR_MODE:
@@ -2026,11 +2015,11 @@ int sys_ioctl(int fd_num,int prm_num,int cmd)
 	switch(prm_num)
 	{
 		case 0:
-			return vent->fs->ioctl(vent->din,prm_num,cmd);
+			return vent->fs->ioctl(vent->din,vent->block,prm_num,cmd);
 		case 1:
-			return vent->fs->ioctl(vent->din,prm_num,cmd,*arg);
+			return vent->fs->ioctl(vent->din,vent->block,prm_num,cmd,*arg);
 		case 2:
-			return vent->fs->ioctl(vent->din,prm_num,cmd,*arg,*(arg+1));
+			return vent->fs->ioctl(vent->din,vent->block,prm_num,cmd,*arg,*(arg+1));
 		default:
 			return 0;
 	}
@@ -2213,7 +2202,7 @@ int mountPageBackupDev(const char *dev_path)
  * Init process0 file struct.
  * return 0 or error number
  */
-int initProc0Fs()
+int initProc1Fs()
 {
 	int rest;
 
