@@ -24,6 +24,7 @@
 #include<console.h>
 #include<share/times.h>
 #include<net/net.h>
+#include<term.h>
 #include<test.h>
 
 
@@ -272,18 +273,6 @@ PROC *searchProcList(int pid)
 }
 
 
-/*
- * GLOBAL
- * Get lock gate address.
- * parameters : process ID
- * return : lock gate address
- */
-int *getProcListRockGate(int pid)
-{
-	return &procHash[getProcHash(pid)].lockGate;
-}
-
-
 /******************************************************************************************************
  *
  * < ץ饹 >
@@ -294,7 +283,7 @@ extern int active_cpu;		/* ư CPU */
 
 /* PUBLIC */
 static int procLockGate=0;		/* ¾Υץ¤λȻΥåȡ */
-static PROC *proc0;				/* pid0ץ¤Ρ */
+static PROC *proc1;				/* pid1ץ¤Ρ */
 
 
 /*
@@ -433,10 +422,10 @@ static void releaseChildProc(PROC *proc)
 	enter_spinlock(&procLockGate);
 	{
 		for(p=proc->child;p!=NULL;p=p->brother)
-			p->parent=proc0;
+			p->parent=proc1;
 		q=releaseChild(proc);
-		q->brother=proc0->child;
-		proc0->child=proc->child;
+		q->brother=proc1->child;
+		proc1->child=proc->child;
 	}
 	exit_spinlock(&procLockGate);
 }
@@ -506,9 +495,9 @@ static int searchExitState(PROC *proc,int pid,int gid,int state,int *stat_loc)
 
 /*
  * GLOBAL
- * proc0wait
+ * proc1wait
  */
-void waitProc0()
+void waitProc1()
 {
 	for(;;)
 	{
@@ -518,7 +507,7 @@ void waitProc0()
 
 		enter_spinlock(&procLockGate);
 		{
-			releaseChild(proc0);
+			releaseChild(proc1);
 		}
 		exit_spinlock(&procLockGate);
 	}
@@ -535,19 +524,22 @@ void waitProc0()
 /*
  * PRIVATE
  * ץ롼פθ
- * parameters : gid,search start process
+ * parameters : search start process,gid
  * return : process or Failed=NULL
  */
-static PROC *searchPg(int gid,PROC *proc)
+static PROC *searchPg(PROC *proc,int gid)
 {
 	PROC *rest;
 
 
 	do
 	{
-		if(proc->gid==gid)return proc;
-		if(proc->child!=NULL)
-			if((rest=searchPg(gid,proc->child))!=NULL)return rest;
+		if(proc->state!=TASK_EXIT)
+		{
+			if(proc->gid==gid)return proc;
+			if(proc->child!=NULL)
+				if((rest=searchPg(proc->child,gid))!=NULL)return rest;
+		}
 	}while((proc=proc->brother)!=NULL);
 
 	return NULL;
@@ -563,7 +555,7 @@ static PROC *searchPg(int gid,PROC *proc)
  */
 static inline int getPid(PROC *proc)
 {
-	static int pid=0;
+	static int pid=1;
 
 	return pid++;
 }
@@ -572,25 +564,23 @@ static inline int getPid(PROC *proc)
 /*
  * GLOBAL
  * Ϳ줿ץ롼פΥץ롣
- * parameters : groupe ID,search start process
+ * parameters : search start process or NULL=proc1,groupe ID
  * return : prosecc or Failed=NULL
  */
-PROC *getNextProc(int gid,PROC *proc)
+PROC *getNextProc(PROC *proc,int gid)
 {
-	if(proc==NULL)
-		/* proc0õ */
-		return searchPg(gid,proc0);
+	/* proc1õ */
+	if(proc==NULL)proc=proc1;
+
+	/* μν֤Υץõ */
+	if(proc->child!=NULL)return searchPg(proc->child,gid);
 	else
 	{
-		/* μν֤Υץõ */
-		if(proc->child!=NULL)return searchPg(gid,proc->child);
-		else
+		do
 		{
-			do
-			{
-				if(proc->brother!=NULL)return searchPg(gid,proc->brother);
-			}while((proc=proc->parent)!=NULL);
-		}
+			if(proc->brother!=NULL)return searchPg(proc->brother,gid);
+			if((proc=proc->parent)==NULL)break;
+		}while(proc->gid==gid);
 	}
 
 	return NULL;
@@ -600,13 +590,13 @@ PROC *getNextProc(int gid,PROC *proc)
 /*
  * GLOBAL
  * Ϳ줿ץμΥץ롣
- * parameters : groupe ID,search start process
+ * parameters : search start process or NULL=proc1
  * return : prosecc or Failed=NULL
  */
 PROC *getAllProc(PROC *proc)
 {
-	/* proc0õ */
-	if(proc==NULL)proc=proc0;
+	/* proc1õ */
+	if(proc==NULL)proc=proc1;
 
 	/* μν֤Υץ */
 	if(proc->child!=NULL)return proc->child;
@@ -949,6 +939,9 @@ int sys_fork()
 	child->gid=parent->gid;
 	child->uid=0;
 
+	/* å󥢥ɥ쥹(ü¤Υɥ쥹)򥳥ԡ */
+	child->ctlterm=parent->ctlterm;
+
 	/* եط¤Τγơ */
 	if((child->file_struct=cpy_file_struct(parent->file_struct))==NULL)goto ERR;
 
@@ -974,8 +967,7 @@ int sys_fork()
 
 	++parent->count_child;
 
-	return (int)child;
-
+	return child->pid;
 ERR:
 	kfree(child);
 
@@ -1036,9 +1028,11 @@ void exit(int state,int signum)
 	void *mm_struct;
 	PROC *proc=get_current_task();
 	PROC *parent;
-	PROC *p;
 
 
+	/* üץγ */
+	releaseTerm(proc);
+
 	/* ץꥹȤ롣 */
 	delProcList(proc);
 
@@ -1048,12 +1042,13 @@ void exit(int state,int signum)
 	parent=proc->parent;
 
 	/* ץ󥯤롣 */
-	if((p=parent->child)==proc)parent->child=proc->brother;
+/*	if((p=parent->child)==proc)parent->child=proc->brother;
 	else
 	{
 		for(;p->brother!=proc;p=p->brother);
 		p->brother=proc->brother;
 	}
+*/
 	--parent->count_child;
 
 	/* եǥץγ */
@@ -1077,7 +1072,7 @@ void exit(int state,int signum)
 	enter_spinlock(&procLockGate);
 	{
 		exitFromSchedule(proc);
-		if(parent->state&TASK_CHILD_WAIT)
+		if((isForegrouond(proc))&&(parent->state&TASK_CHILD_WAIT))
 			add_to_schedule(parent,TASK_SIGNAL_WAIT);
 	}
 	exit_spinlock(&procLockGate);
@@ -1140,7 +1135,10 @@ int sys_get_procid(int pid,PROC_ID *pd)
 	if(pid==0)
 		proc=get_current_task();
 	else
+	{
 		if((proc=searchProcList(pid))==NULL)return -ESRCH;
+		if(proc->ctlterm!=get_current_task()->ctlterm)return -EPERM;
+	}
 
 	pd->pid=proc->pid;
 	pd->gid=proc->gid;
@@ -1162,13 +1160,20 @@ int sys_set_procid(int pid,PROC_ID *pd)
 	if(pid==0)
 		proc=get_current_task();
 	else
+	{
 		if((proc=searchProcList(pid))==NULL)return -ESRCH;
+	}
 
 	if(pd->gid>=0)
 	{
-		if(proc->gid!=0)return -EPERM;
-		if(proc->pid==pd->gid)proc->gid=pd->gid;
-		else return -EPERM;
+		/* ץå꡼ */
+		if(isCtlproc(proc))return -EPERM;
+
+		/* ȥץλҥץǤʡ */
+		if(proc->gid!=get_current_task()->gid)return -EPERM;
+
+		if(pd->gid==0)proc->gid=proc->pid;
+		else proc->gid=pd->gid;
 	}
 	else if(pd->uid>=0)
 	{
@@ -1196,6 +1201,17 @@ int sys_times(struct tms *tms)
 }
 
 
+/*
+ * ü򥯥ꥢ롣
+ *
+ */
+int sys_setsid()
+{
+	get_current_task()->ctlterm=NULL;
+	return 0;
+}
+
+
 /******************************************************************************************************
  *
  * < 饹 >
@@ -1276,12 +1292,12 @@ int set_idle_proc(int cpu)
  * ǽΥץꡣ
  * return : 0 or error number
  */
-int initProc0()
+int initProc1()
 {
-	proc0=get_current_task();
-	proc0->parent=NULL;
+	proc1=get_current_task();
+	proc1->parent=NULL;
 
-	return initProc0Fs();
+	return initProc1Fs();
 }
 
 /*************************************************************************************************/
