@@ -22,7 +22,7 @@ extern void test_orig_fs();
 extern void test_fatFs();
 extern void test_elf(void*);
 extern int test_ext2();
-extern void test_mm();
+extern int test_mm();
 extern void test_proc();
 extern void test_pci();
 extern int test_arp();
