@@ -7,6 +7,10 @@
 #define share_stat_h
 
 
+#include<share/types.h>
+#include<share/stdint.h>
+
+
 #ifndef ASM_FILE
 
 
@@ -23,6 +27,7 @@ struct stat{
 	time_t    st_mtime;		/* Time of last data modification. */
 	time_t    st_ctime;		/* Time of last status change. */
 	size_t    st_crtpos;	/* option,current file position. */
+	uint32_t  st_blksize;	/* optimal blocksize for I/O */
 };
 
 
