@@ -16,7 +16,7 @@ enum{
 	/* Version Test Symbolic Constants. */
 	_POSIX_VERSION=199009L,
 	_POSIX2_VERSION=200112L,
-	_POSIX_JOB_CONTROL=0,
+	_POSIX_JOB_CONTROL=1,
 	_POSIX_SAVED_IDS=0,
 
 	/* Execution-Time Symbolic Constants. */
@@ -171,6 +171,10 @@ enum{
 };
 
 
+extern char *optarg;
+extern int optind, opterr, optopt;
+
+
 /* Function declarations. */
 extern int          access(const char *, int);
 extern unsigned     alarm(unsigned);
@@ -193,6 +197,7 @@ extern gid_t        getegid(void);
 extern uid_t        geteuid(void);
 extern gid_t        getgid(void);
 extern char        *getlogin(void);
+extern pid_t        getpgid(pid_t);
 extern pid_t        getpid(void);
 extern int          getgroups(int, gid_t []);
 extern pid_t        getpgrp(void);
@@ -219,11 +224,14 @@ extern int          tcsetpgrp(int, pid_t);
 extern char        *ttyname(int);
 extern ssize_t      write(int, const void *, size_t);
 extern int          unlink(const char *);
+extern int          getopt(int argc, char * const argv[], const char *optstring);
 
 
 /* ץޥ */
 #define ROUNDUP(a,b) (((uint)(a)+(b)-1)/(b)*(b))
 #define ROUNDDOWN(a,b) ((uint)(a)/(b)*(b))
+#define	MIN(a,b) (((a)<(b))?(a):(b))
+#define	MAX(a,b) (((a)>(b))?(a):(b))
 
 /* ץؿ */
 extern int brk(void*);
