@@ -1188,8 +1188,8 @@ OCTAL:				if(flag&SFLG_DETER)
  ***************************************************************************/
 
 enum{
-	F_FLAG_EOF=1<<0,	/* File sterm EOF flag. */
-	F_FLAG_ERR=1<<1,	/* File sterm error flag. */
+	F_FLAG_EOF=1<<0,	/* File stream EOF flag. */
+	F_FLAG_ERR=1<<1,	/* File stream error flag. */
 };
 
 
@@ -1198,7 +1198,7 @@ FILE _stdout={1,0,0,0,NULL};
 FILE _stderr={2,0,0,0,NULL};
 FILE *stdin	=&_stdin;
 FILE *stdout=&_stdout;
-FILE *strerr=&_stderr;
+FILE *stderr=&_stderr;
 
 
 /*
@@ -1408,6 +1408,12 @@ size_t fwrite(const void* ptr,size_t size,size_t nobj,FILE* stream)
 }
 
 
+int fileno(FILE *stream)
+{
+	return stream->fd;
+}
+
+
 int setvbuf(FILE *stream, char* buf, int mode, size_t size)
 {
 	switch(mode)
@@ -1703,7 +1709,7 @@ int sscanf(const char* s, const char* format, ...)
 	va_list ap;
 
 
-	va_start(ap,format);
+	va_start(ap,(format));
 	return formatFromString(s,format,(uint**)ap);
 }
 
